﻿using System;
using System.Collections.Generic;

namespace kujutils
{
    public class ActionGroupThrottleLimit : PartModule
    {
        public struct EnginePair
        {
            public ModuleEngines engine;
            public float limit;

            public EnginePair(ModuleEngines e)
            {
                engine = e;
                limit = 100;
            }
        }
        public struct EngineFXPair
        {
            public ModuleEnginesFX engine;
            public float limit;

            public EngineFXPair(ModuleEnginesFX e)
            {
                engine = e;
                limit = 100;
            }
        }

        public List<EnginePair> engines;
        public List<EngineFXPair> engineFXs;

        [KSPField(guiName = "Limiter", isPersistant = true, guiActive = true),
            UI_Toggle(disabledText = "Disabled",
            scene = UI_Scene.All,
            enabledText = "Enabled",
            affectSymCounterparts = UI_Scene.Editor)]
        public bool throttleLimited = false;

        [KSPField(guiName = "Action Throttle Limit", isPersistant = true, guiActive = true),
            UI_FloatRange(minValue = 0f,
            stepIncrement = 1f,
            maxValue = 100f,
            affectSymCounterparts = UI_Scene.All)]
        public float throttleLimit = 100f;

        [KSPAction("Toggle Throttle Limit")]
        public void ToggleAction(KSPActionParam param)
        {
            if (throttleLimited)
            {
                ReleaseAction(param);
            }
            else
            {
                SetAction(param);
            }
        }

        [KSPAction("Set Limiter")]
        public void SetAction(KSPActionParam param)
        {
            this.throttleLimited = true;
            print("There are " + engines.Count + " engines");

            for (int i = 0; i < engines.Count; i++)
            {
                var e = engines[i];
                e.limit = engines[i].engine.thrustPercentage;
                engines[i].engine.thrustPercentage = throttleLimit;
            }
            for (int i = 0; i < engineFXs.Count; i++)
            {
                var e = engineFXs[i];
                e.limit = engineFXs[i].engine.thrustPercentage;
                engineFXs[i].engine.thrustPercentage = throttleLimit;
            }
        }

        [KSPAction("Release Limiter")]
        public void ReleaseAction(KSPActionParam param)
        {
            this.throttleLimited = false;
            foreach (EnginePair e in engines)
            {
                e.engine.thrustPercentage = e.limit;
            }
            foreach (EngineFXPair e in engineFXs)
            {
                e.engine.thrustPercentage = e.limit;
            }
        }

        public override void OnFixedUpdate()
        {
            //if (throttleLimited) LimitThrottle();
        }

        private void LimitThrottle()
        {
            if (throttleLimit > 0.995f) return;

            var craftThrottle = this.vessel.ctrlState.mainThrottle;

            foreach (EnginePair e in engines)
            {
                e.engine.thrustPercentage = throttleLimit;
            }
            foreach (EngineFXPair e in engineFXs)
            {
                e.engine.thrustPercentage = throttleLimit;
            }
        }

        private void BuildEngineArrays()
        {
            engines = new List<EnginePair>();
            engineFXs = new List<EngineFXPair>();

            foreach(ModuleEngines e in this.part.Modules.GetModules<ModuleEngines>())
            {
                var ep = new EnginePair(e);
                engines.Add(ep);
            }

            foreach (ModuleEnginesFX e in this.part.Modules.GetModules<ModuleEnginesFX>())
            {
                var ep = new EngineFXPair(e);
                engineFXs.AddUnique(ep);
            }
        }

        public override void OnStart(PartModule.StartState state)
        {
            BuildEngineArrays();
        }

        public override void OnLoad(ConfigNode node)
        {
            BuildEngineArrays();
        }

    }
}

